import 'dart:io';

import 'package:apk_update/core/common/version_info.dart';
import 'package:apk_update/core/services/version_service/version_service.dart';
import 'package:flustars_flutter3/flustars_flutter3.dart';

import '../../../../utils/dio_update_util.dart';
import '../../../../utils/toast_util.dart';
import '../../../common/app_upgrade_config.dart';
import '../../../common/constants.dart';
import '../../../common/servier_api.dart';

/// 项目管理工具
class PMVersionImpl implements VersionService {
  final AppUpgradeConfig config;

  const PMVersionImpl({required this.config});

  @override
  Future<VersionInfo?> checkVersion() async {
    // ①版本检测
    var checkResult = await checkVersionPM();
    if (checkResult == null) return null;
    // ②获取版本详情
    var result = await getVersionDetailPM(checkResult?['id']);
    if (result == null) return null;
    //获取当前时间
    String spCurrent = SpUtil.getString(Constants.spCurrentDay) ?? "";
    String current =
        DateUtil.formatDate(DateTime.now(), format: Constants.dateFormat);
    //一天只提醒一次：一天提示一次升级标识 & 非强升 & 当天已提示 => 不再提示
    if (config.onceDay &&
        result?['forceUpgradeStatus'] == 0 &&
        spCurrent == current) {
      // 不再提示
      return null;
    }
    //缓存当前时间
    SpUtil.putString(Constants.spCurrentDay, current);
    // 是否是最新版本提示
    final newVersion = int.tryParse(result?['version']?.toString() ?? '') ?? 0;
    final currentVersion =
        int.tryParse(config.versionNumber?.toString() ?? '') ?? 0;
    if (newVersion <= currentVersion) {
      if (!config.onceDay) {
        ToastUtil.showToast('已经是最新版本了！');
      }
      return null;
    }
    // 返回升级版本信息
    return VersionInfo(
      name: result['name'],
      content: result['popContent'],
      versionForce: result['forceUpgradeStatus'] == 1,
      versionPath: result['versionPath'],
    );
  }

  /// 版本检测
  Future<dynamic> checkVersionPM() async {
    var checkResult = await DioUpdateUtil.get(
      "${config.baseUrl}${ServerApi.getLatestByProductCodePM}",
      queryParameters: {
        "productCode": config.productCode,
        "currentVersion": config.versionNumber,
        "sideType": Platform.isIOS ? 2 : 1
      },
      headers: {"productGroupCode": "common"},
    );
    if (checkResult == null) return null;
    return checkResult;
  }

  /// 获取版本详情
  Future<dynamic> getVersionDetailPM(int? id) async {
    var detailResult = await DioUpdateUtil.get(
        "${config.baseUrl}${ServerApi.getDetailPM}",
        queryParameters: {'id': id},
        headers: {"productGroupCode": "common"});
    if (detailResult == null) return null;
    return detailResult;
  }
}
